@@ -1,10 +1,10 @@
 /*
 
-Copyright (c) 2010-2020, Arvid Norberg
+Copyright (c) 2010-2022, Arvid Norberg
 Copyright (c) 2015, Mike Tzou
-Copyright (c) 2016-2017, Steven Siloti
-Copyright (c) 2016, Andrei Kurushin
 Copyright (c) 2016, 2018, Alden Torres
+Copyright (c) 2016, Andrei Kurushin
+Copyright (c) 2016-2017, Steven Siloti
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -916,7 +916,7 @@ void write_handler(file_storage const& fs
 	disk.submit_jobs();
 }
 
-void generate_data(char const* path, torrent_info const& ti)
+void generate_data(std::string const path, torrent_info const& ti)
 {
 	io_context ios;
 	counters stats_counters;
