@@ -885,7 +885,7 @@ TORRENT_TEST(pex)
 	lt::deadline_timer timer(ios);
 
 	lt::error_code ec;
-	int const swarm_id = test_counter();
+	int const swarm_id = unit_test::test_counter();
 	std::string path = save_path(swarm_id, 0);
 
 	lt::create_directory(path, ec);
