@@ -1,8 +1,9 @@
 /*
 
-Copyright (c) 2016, 2019-2020, Arvid Norberg
+Copyright (c) 2016, 2019-2022, Arvid Norberg
 Copyright (c) 2019, Steven Siloti
 Copyright (c) 2020, Tiger Wang
+Copyright (c) 2021, Alden Torres
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -61,6 +62,10 @@ auto const map_failed = MAP_FAILED;
 #include "libtorrent/aux_/disable_warnings_pop.hpp"
 #endif
 
+#if TORRENT_USE_SYNC_FILE_RANGE
+#include <fcntl.h> // for sync_file_range
+#endif
+
 namespace libtorrent {
 namespace aux {
 
@@ -229,6 +234,18 @@ namespace {
 
 	auto create_file(const native_path_string & name, open_mode_t const mode)
 	{
+
+		if (mode & aux::open_mode::allow_set_file_valid_data)
+		{
+			// Enable privilege required by SetFileValidData()
+			// https://docs.microsoft.com/en-us/windows/desktop/api/fileapi/nf-fileapi-setfilevaliddata
+			// This must happen before the file is opened:
+			// https://devblogs.microsoft.com/oldnewthing/20160603-00/?p=93565
+			// SetFileValidData() is not available on WinRT, so there is no
+			// corresponding call in that version of create_file()
+			std::call_once(g_once_flag, acquire_manage_volume_privs);
+		}
+
 		return CreateFileW(name.c_str()
 			, file_access(mode)
 			, FILE_SHARE_WRITE | FILE_SHARE_READ | FILE_SHARE_DELETE
@@ -274,10 +291,6 @@ file_handle::file_handle(string_view name, std::int64_t const size
 			throw_ex<storage_error>(error_code(GetLastError(), system_category()), operation_t::file_truncate);
 
 #ifndef TORRENT_WINRT
-		// Enable privilege required by SetFileValidData()
-		// https://docs.microsoft.com/en-us/windows/desktop/api/fileapi/nf-fileapi-setfilevaliddata
-		std::call_once(g_once_flag, acquire_manage_volume_privs);
-
 		// if the user has permissions, avoid filling
 		// the file with zeroes, but just fill it with
 		// garbage instead
@@ -728,8 +741,13 @@ void file_mapping::page_out(span<byte const> range)
 	auto const size = static_cast<std::size_t>(range.size());
 #if TORRENT_USE_MADVISE && defined MADV_PAGEOUT
 	::madvise(start, size, MADV_PAGEOUT);
+#elif TORRENT_USE_SYNC_FILE_RANGE
+	// this is best-effort. ignore errors
+	::sync_file_range(m_file.fd(), start - static_cast<const byte*>(m_mapping)
+		, size, SYNC_FILE_RANGE_WRITE);
 #endif
 
+	// msync(MS_ASYNC) is a no-op on Linux > 2.6.19.
 	::msync(start, size, MS_ASYNC);
 
 #endif // MAP_VIEW_OF_FILE
