@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2016, 2019-2020, Arvid Norberg
+Copyright (c) 2016, 2019-2022, Arvid Norberg
 Copyright (c) 2017-2018, Steven Siloti
 Copyright (c) 2020, Alden Torres
 All rights reserved.
@@ -60,7 +60,6 @@ namespace {
 
 	struct TORRENT_EXTRA_EXPORT posix_disk_io final
 		: disk_interface
-		, buffer_allocator_interface
 	{
 		posix_disk_io(io_context& ios, settings_interface const& sett, counters& cnt)
 			: m_settings(sett)
@@ -100,13 +99,13 @@ namespace {
 			, std::function<void(disk_buffer_holder block, storage_error const& se)> handler
 			, disk_job_flags_t) override
 		{
-			disk_buffer_holder buffer = disk_buffer_holder(*this, m_buffer_pool.allocate_buffer("send buffer"), default_block_size);
+			disk_buffer_holder buffer = disk_buffer_holder(m_buffer_pool, m_buffer_pool.allocate_buffer("send buffer"), default_block_size);
 			storage_error error;
 			if (!buffer)
 			{
 				error.ec = errors::no_memory;
 				error.operation = operation_t::alloc_cache_piece;
-				post(m_ios, [=, h = std::move(handler)]{ h(disk_buffer_holder(*this, nullptr, 0), error); });
+				post(m_ios, [=, h = std::move(handler)]{ h(disk_buffer_holder(m_buffer_pool, nullptr, 0), error); });
 				return;
 			}
 
@@ -168,7 +167,7 @@ namespace {
 			bool const v1 = bool(flags & disk_interface::v1_hash);
 			bool const v2 = !block_hashes.empty();
 
-			disk_buffer_holder buffer = disk_buffer_holder(*this, m_buffer_pool.allocate_buffer("hash buffer"), default_block_size);
+			disk_buffer_holder buffer = disk_buffer_holder(m_buffer_pool, m_buffer_pool.allocate_buffer("hash buffer"), default_block_size);
 			storage_error error;
 			if (!buffer)
 			{
@@ -228,7 +227,7 @@ namespace {
 		{
 			time_point const start_time = clock_type::now();
 
-			disk_buffer_holder buffer = disk_buffer_holder(*this, m_buffer_pool.allocate_buffer("hash buffer"), 0x4000);
+			disk_buffer_holder buffer = disk_buffer_holder(m_buffer_pool, m_buffer_pool.allocate_buffer("hash buffer"), 0x4000);
 			storage_error error;
 			if (!buffer)
 			{
@@ -308,28 +307,30 @@ namespace {
 			storage_error error;
 			status_t const ret = [&]
 			{
-				st->initialize(m_settings, error);
-				if (error) return status_t::fatal_disk_error;
+				auto const ret_flag = st->initialize(m_settings, error);
+				if (error) return status_t::fatal_disk_error | ret_flag;
 
 				bool const verify_success = st->verify_resume_data(*rd
 					, std::move(links), error);
 
 				if (m_settings.get_bool(settings_pack::no_recheck_incomplete_resume))
-					return status_t::no_error;
+					return status_t::no_error | ret_flag;
 
 				if (!aux::contains_resume_data(*rd))
 				{
 					// if we don't have any resume data, we still may need to trigger a
 					// full re-check, if there are *any* files.
 					storage_error ignore;
-					return (st->has_any_file(ignore))
+					return ((st->has_any_file(ignore))
 						? status_t::need_full_check
-						: status_t::no_error;
+						: status_t::no_error)
+						| ret_flag;
 				}
 
-				return verify_success
+				return (verify_success
 					? status_t::no_error
-					: status_t::need_full_check;
+					: status_t::need_full_check)
+					| ret_flag;
 			}();
 
 			post(m_ios, [error, ret, h = std::move(handler)]{ h(ret, error); });
@@ -371,10 +372,6 @@ namespace {
 			post(m_ios, [=, h = std::move(handler)]{ h(index); });
 		}
 
-		// implements buffer_allocator_interface
-		void free_disk_buffer(char* b) override
-		{ m_buffer_pool.free_buffer(b); }
-
 		void update_stats_counters(counters&) const override {}
 
 		std::vector<open_file_state> get_status(storage_index_t) const override
