@@ -1,9 +1,9 @@
 /*
 
-Copyright (c) 2006-2020, Arvid Norberg
+Copyright (c) 2006-2022, Arvid Norberg
 Copyright (c) 2015-2017, Alden Torres
-Copyright (c) 2016-2018, Steven Siloti
 Copyright (c) 2016, Andrei Kurushin
+Copyright (c) 2016-2018, Steven Siloti
 Copyright (c) 2017, 2020, AllSeeingEyeTolledEweSew
 Copyright (c) 2018, d-komarov
 All rights reserved.
@@ -179,7 +179,7 @@ void add_mp(span<std::uint8_t> target, span<std::uint8_t const> add)
 	{
 		int const res = carry + int(target[i]) + add[i];
 		carry = res >> 8;
-		target[i] = res % 255;
+		target[i] = std::uint8_t(res & 255);
 	}
 }
 }
@@ -716,7 +716,7 @@ std::vector<std::string> get_python()
 
 int find_available_port()
 {
-	int port = 2000 + (std::int64_t(::getpid()) + _g_test_idx + std::rand()) % 60000;
+	int port = 2000 + (std::int64_t(::getpid()) + ::unit_test::g_test_idx + std::rand()) % 60000;
 	error_code ec;
 	io_context ios;
 
@@ -894,12 +894,10 @@ void create_random_files(std::string const& path, span<const int> file_sizes
 		int to_write = file_sizes[i];
 		if (fs) fs->add_file(full_path, to_write);
 		ofstream f(full_path.c_str());
-		std::int64_t offset = 0;
 		while (to_write > 0)
 		{
 			int const s = std::min(to_write, static_cast<int>(random_data.size()));
 			f.write(random_data.data(), s);
-			offset += s;
 			to_write -= s;
 		}
 	}
