@@ -1,7 +1,7 @@
 /*
 
-Copyright (c) 2016, Alden Torres
-Copyright (c) 2016-2020, Arvid Norberg
+Copyright (c) 2016, 2021, Alden Torres
+Copyright (c) 2016-2022, Arvid Norberg
 Copyright (c) 2016, Pavel Pimenov
 Copyright (c) 2017-2018, Steven Siloti
 All rights reserved.
@@ -60,11 +60,6 @@ TORRENT_TEST(create_verbatim_torrent)
 		"12:piece lengthi65536e6:pieces20:ababababababababababee";
 
 	lt::torrent_info info(test_torrent, lt::from_span);
-
-	info.add_tracker("http://test.com");
-	info.add_tracker("http://test.com");
-	TEST_EQUAL(info.trackers().size(), 1);
-
 	lt::create_torrent t(info);
 
 	std::vector<char> buffer;
