@@ -1,10 +1,10 @@
 /*
 
-Copyright (c) 2014-2020, Arvid Norberg
+Copyright (c) 2014-2022, Arvid Norberg
 Copyright (c) 2015, Jakob Petsovits
 Copyright (c) 2016, Eugene Shalygin
-Copyright (c) 2017, Steven Siloti
 Copyright (c) 2017-2018, Alden Torres
+Copyright (c) 2017, Steven Siloti
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -117,7 +117,7 @@ void test_remap_files(storage_mode_t storage_mode = storage_mode_sparse)
 	for (auto const i : fs.piece_range())
 	{
 		std::vector<char> const piece = generate_piece(i, fs.piece_size(i));
-		tor1.add_piece(i, piece.data());
+		tor1.add_piece(i, std::move(piece));
 	}
 
 	// read pieces
